from django_lare import VERSION


class Lare(object):

    enabled = False
    current_namespace = ""
    previous_namespace = ""
    version = VERSION
    supported_version = "1.0.0"

    def __init__(self, request):
        super(Lare, self).__init__()
        if 'HTTP_X_LARE' in request.META:
            if 'HTTP_X_LARE_VERSION' in request.META:
                frontend_version = request.META['HTTP_X_LARE_VERSION']
                frontend_versions = frontend_version.split('.')
                supported_versions = self.supported_version.split('.')
                i = 0
                while i < len(supported_versions):
                    if frontend_versions[i] < supported_versions[i]:
                        self.enabled = False
                        return
                    i += 1
            self.enabled = True
            self.previous_namespace = request.META['HTTP_X_LARE']

    def set_current_namespace(self, namespace):
        self.current_namespace = namespace

    def get_current_namespace(self):
        return self.current_namespace

    def is_enabled(self):
        return self.enabled

    def get_matching_count(self, extension_namespace=None):
        if not self.enabled:
            return 0
        if extension_namespace is None:
            extension_namespace = self.current_namespace

        matching_count = 0
        previous_namespaces = self.previous_namespace.split('.')
        extension_namespaces = extension_namespace.split('.')

        while matching_count < len(previous_namespaces) and matching_count < len(extension_namespaces):
            if previous_namespaces[matching_count] == extension_namespaces[matching_count]:
                matching_count += 1
            else:
                break
        return matching_count

    def matches(self, extension_namespace=None):
        if extension_namespace is None:
            extension_namespace = self.current_namespace
        return self.get_matching_count(extension_namespace) == len(extension_namespace.split('.'))



# Django settings for testproject project.
import os

PROJECT_ROOT = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))

DEBUG = True
TEMPLATE_DEBUG = DEBUG
DEBUG_PROPAGATE_EXCEPTIONS = True

ALLOWED_HOSTS = ['*']

ADMINS = ()

MANAGERS = ADMINS

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',  # Add 'postgresql_psycopg2', 'postgresql', 'mysql', 'sqlite3' or 'oracle'.
        'NAME': 'sqlite.db',                     # Or path to database file if using sqlite3.
        'USER': '',                      # Not used with sqlite3.
        'PASSWORD': '',                  # Not used with sqlite3.
        'HOST': '',                      # Set to empty string for localhost. Not used with sqlite3.
        'PORT': '',                      # Set to empty string for default. Not used with sqlite3.
    }
}

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
    }
}

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# On Unix systems, a value of None will cause Django to use the same
# timezone as the operating system.
# If running in a Windows environment this must be set to the same as your
# system time zone.
TIME_ZONE = 'Europe/London'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-uk'

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# If you set this to False, Django will not format dates, numbers and
# calendars according to the current locale
USE_L10N = True

# Absolute filesystem path to the directory that will hold user-uploaded files.
# Example: "/home/media/media.lawrence.com/"
MEDIA_ROOT = ''

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash if there is a path component (optional in other cases).
# Examples: "http://media.lawrence.com", "http://example.com/media/"
MEDIA_URL = ''

# Make this unique, and don't share it with anybody.
SECRET_KEY = 'u@x-aj9(hoh#rb-^ymf#g2jx_hp0vj7u5#b@ag1n^seu9e!%cy'

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
)

ROOT_URLCONF = 'tests'

TEMPLATE_DIRS = (
    # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
)

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
)

STATIC_URL = '/static/'

PASSWORD_HASHERS = (
    'django.contrib.auth.hashers.SHA1PasswordHasher',
    'django.contrib.auth.hashers.PBKDF2PasswordHasher',
    'django.contrib.auth.hashers.PBKDF2SHA1PasswordHasher',
    'django.contrib.auth.hashers.BCryptPasswordHasher',
    'django.contrib.auth.hashers.MD5PasswordHasher',
    'django.contrib.auth.hashers.CryptPasswordHasher',
)

AUTH_USER_MODEL = 'auth.User'

import django

if django.VERSION < (1, 3):
    INSTALLED_APPS += ('staticfiles',)

# django lare
INSTALLED_APPS += ('django_lare', )

MIDDLEWARE_CLASSES += (
    'django_lare.middlewares.LareMiddleware',
)
if django.VERSION >= (1, 7):
    import django
    django.setup()
if django.VERSION <= (1, 9):
    from django.template.base import add_to_builtins
    add_to_builtins('django_lare.templatetags.lare_extends',)
    TEMPLATE_CONTEXT_PROCESSORS = ('django_lare.context_processors.lare_information',)
else:
    TEMPLATES = [{
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            'django_lare/templates',
        ],
        'OPTIONS': {
            'context_processors': [
                'django_lare.context_processors.lare_information',
            ],
            'builtins': [
                'django_lare.templatetags.lare_extends',
            ],
        },
    }]

DEFAULT_LARE_TEMPLATE = "django_lare/lare.html"



import os
import sys


# adjusting sys.path
path = os.path.abspath(os.path.join(os.path.abspath(os.path.dirname(__file__)), '..', '..'))
sys.path.append(path)

os.environ['DJANGO_SETTINGS_MODULE'] = 'django_lare.test_settings'


import django
from django.conf import settings
from django.test.utils import get_runner


def usage():
    return """
    Usage: python runtests.py [UnitTestClass].[method]

    You can pass the Class name of the `UnitTestClass` you want to test.

    Append a method name if you only want to test a specific method of that class.
    """


def main():
    TestRunner = get_runner(settings)

    test_runner = TestRunner()

    failures = test_runner.run_tests(['django_lare'])

    sys.exit(failures)


if __name__ == '__main__':
    main()



from django.conf import settings
from django.template.base import TemplateSyntaxError, FilterExpression
from django.template import Library
from django.template.loader import get_template  # import to solve ImportErrors
from django.template.loader_tags import ExtendsNode

register = Library()


class LareExtendsNode(ExtendsNode):
    def __init__(self, nodelist, parent_name, lare_namespace, lare_template, template_dirs=None):
        super(LareExtendsNode, self).__init__(nodelist, parent_name, template_dirs=template_dirs)
        self.lare_namespace = lare_namespace
        self.lare_template = lare_template

    def __repr__(self):
        return '<LareExtendsNode: extends %s>' % self.parent_name.token

    def get_parent(self, context):
        lare_context = dict((k, v) for d in context.dicts for k, v in d.items() if (k == 'lare'))
        lare = lare_context.get('lare', False)
        if lare and lare.is_enabled() and lare.matches(self.lare_namespace.resolve(context)):
                self.parent_name = self.lare_template
            # try:
            #     namespace = lare.get_current_namspace()
            # except KeyError:
            #     pass  # no namespace given, so do not change parent_name => initial request
            # else:
            #     if namespace.startswith(self.lare_namespace.resolve(context)):
            #         self.parent_name = self.lare_template

        return super(LareExtendsNode, self).get_parent(context)


@register.tag()
def lare_extends(parser, token):
    bits = token.split_contents()
    if len(bits) != 4 and len(bits) != 3 and len(bits) != 2:
        raise TemplateSyntaxError("'%s' takes 1 - 3 arguments" % bits[0])

    nodelist = parser.parse()

    if nodelist.get_nodes_by_type(LareExtendsNode) or nodelist.get_nodes_by_type(ExtendsNode):
        raise TemplateSyntaxError("'lare_extends' and 'extends' cannot appear more than once in the same template!")

    if len(bits) > 2:
        try:
            # format DEFAULT_LARE_TEMPLATE string to fit into FilterExpression as token
            lare_template = parser.compile_filter(bits[3]) if (len(bits) == 4) else FilterExpression("'{0}'".format(settings.DEFAULT_LARE_TEMPLATE), parser)
        except AttributeError:
            raise TemplateSyntaxError("No lare template set, even no default!")
        return LareExtendsNode(nodelist, parser.compile_filter(bits[1]), parser.compile_filter(bits[2]), lare_template)
    return ExtendsNode(nodelist, parser.compile_filter(bits[1]))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup
import re
import os
import sys


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.match("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


def get_packages(package):
    """
    Return root package and all sub-packages.
    """
    return [dirpath
            for dirpath, dirnames, filenames in os.walk(package)
            if os.path.exists(os.path.join(dirpath, '__init__.py'))]


def get_package_data(package):
    """
    Return all files under the root package, that are not in a
    package themselves.
    """
    walk = [(dirpath.replace(package + os.sep, '', 1), filenames)
            for dirpath, dirnames, filenames in os.walk(package)
            if not os.path.exists(os.path.join(dirpath, '__init__.py'))]

    filepaths = []
    for base, filenames in walk:
        filepaths.extend([os.path.join(base, filename)
                          for filename in filenames])
    return {package: filepaths}


version = get_version('django_lare')


if sys.argv[-1] == 'publish':
    os.system("python setup.py sdist upload")
    print("You probably want to also tag the version now:")
    print("  git tag -a %s -m 'version %s'" % (version, version))
    print("  git push --tags")
    sys.exit()


setup(name='django-lare',
      version=version,
      author='Lare Team',
      author_email='lare-team@lare.team',
      license='MIT',
      url='http://lare-team.de',
      description='Lare extension to use lare.js easily!',
      packages=get_packages('django_lare'),
      package_data=get_package_data('django_lare'),
      py_modules=['django-lare'],
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Environment :: Web Environment',
          'Framework :: Django',
          'Intended Audience :: Developers',
          'Operating System :: OS Independent',
          'Programming Language :: Python',
          'Programming Language :: Python :: 3',
          'Topic :: Internet :: WWW/HTTP',
      ]
      )
